﻿using System;
using System.Transactions;
using TxClientApp.TxFlowService;

namespace TxClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Simple Transaction Flow Client App......");

			string epConfigName = "WSHttpBinding_ITxFlowService";
			//string epConfigName = "NetTcpBinding_ITxFlowService";

			//---------------- TxMethod1 Test ----------------------
			using (TransactionScope scope = new TransactionScope())
			using (TxFlowServiceClient proxy = new TxFlowServiceClient(epConfigName)) {
				Console.WriteLine("++ Before invoke TxMethod1()...");
				ShowCurrentTransactionInfo();
				proxy.TxMethod1();
				Console.WriteLine("-- After invoke TxMethod1()...");
				ShowCurrentTransactionInfo();
				scope.Complete();
			}

			//---------------- TxMethod2 Test ----------------------
			using (TransactionScope scope = new TransactionScope())
			using (TxFlowServiceClient proxy = new TxFlowServiceClient(epConfigName)) {
				Console.WriteLine("++ Before invoke TxMethod2()... (within Transaction)");
				ShowCurrentTransactionInfo();
				proxy.TxMethod2();
				Console.WriteLine("-- After invoke TxMethod2()... (within Transaction)");
				ShowCurrentTransactionInfo();
				scope.Complete();
			}

			using (TxFlowServiceClient proxy = new TxFlowServiceClient(epConfigName)) {
				Console.WriteLine("++ Before invoke TxMethod2()... (NO Transaction)");
				ShowCurrentTransactionInfo();
				proxy.TxMethod2();
				Console.WriteLine("-- After invoke TxMethod2()... (NO Transaction)");
				ShowCurrentTransactionInfo();
			}

			//---------------- TxMethod3 Test ----------------------
			using (TransactionScope scope = new TransactionScope())
			using (TxFlowServiceClient proxy = new TxFlowServiceClient(epConfigName)) {
				Console.WriteLine("++ Before invoke TxMethod3()... (within Transaction)");
				ShowCurrentTransactionInfo();
				proxy.TxMethod3();
				Console.WriteLine("-- After invoke TxMethod3()... (within Transaction)");
				ShowCurrentTransactionInfo();
				scope.Complete();
			}

			using (TxFlowServiceClient proxy = new TxFlowServiceClient(epConfigName)) {
				Console.WriteLine("++ Before invoke TxMethod3()... (NO Transaction)");
				ShowCurrentTransactionInfo();
				proxy.TxMethod3();
				Console.WriteLine("-- After invoke TxMethod3()... (NO Transaction)");
				ShowCurrentTransactionInfo();
			}
		}

		#region "Tx Info Helper"

		static string GetTransactionId(Transaction tx)
		{
			if (tx.TransactionInformation.DistributedIdentifier != Guid.Empty) {
				return tx.TransactionInformation.DistributedIdentifier.ToString();
			}
			else {
				return tx.TransactionInformation.LocalIdentifier;
			}
		}

		static void ShowCurrentTransactionInfo()
		{
			bool isInTx = (Transaction.Current != null);

			Console.WriteLine("  >> Transaction Information:");
			Console.WriteLine("\tIs in transaction : {0}", isInTx);
			if (isInTx == true) {
				Transaction tx = Transaction.Current;
				TransactionInformation ti = tx.TransactionInformation;
				bool isDtx = (tx.TransactionInformation.DistributedIdentifier != Guid.Empty);
				string txId = GetTransactionId(tx);
				DateTime creationTime = ti.CreationTime;

				Console.WriteLine("\tTransaction type : {0}", isDtx ? "Distributed" : "Local");
				Console.WriteLine("\tTransaction id : {0}", txId);
				Console.WriteLine("\tCreation time : {0}", creationTime);
			}
		}

		#endregion
	}
}
