﻿using System;
using System.Data;
using System.ServiceModel;
using System.Transactions;
using System.Windows.Forms;
using TxClientApp.TxService1;
using TxClientApp.TxService2;

namespace TxClientApp
{
	public partial class TxClientForm : Form
	{
		public TxClientForm()
		{
			InitializeComponent();
		}

		private void GetData()
		{
			dataGridView1.DataSource = null;
			using (TxService1Client proxy = new TxService1Client()) {
				DataSet ds = proxy.GetData();
				dataGridView1.DataSource = ds.Tables[0];
			}

			dataGridView2.DataSource = null;
			using (TxService2Client proxy = new TxService2Client()) {
				DataSet ds = proxy.GetData();
				dataGridView2.DataSource = ds.Tables[0];
			}
		}

		private void InsertData()
		{
			bool rollbackClient = checkBox1.Checked;
			bool throwException1 = checkBox2.Checked;
			bool throwExceptoin2 = checkBox3.Checked;

			try {
				using (TransactionScope scope = new TransactionScope()) {
					using (TxService1Client proxy = new TxService1Client()) {
						proxy.InsertData(1);
					}

					using (TxService2Client proxy = new TxService2Client()) {
						proxy.InsertData(2);
					}

					if (rollbackClient) {
						throw new InvalidOperationException("Client Error for Test.");
					}

					scope.Complete();
				}
			}
			catch (FaultException ex) {
				MessageBox.Show(ex.Message, "Transaction Error in Service",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			catch (InvalidOperationException ex) {
				MessageBox.Show(ex.Message, "Transaction Error in Client",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void btnInsertData_Click(object sender, EventArgs e)
		{
			InsertData();
			GetData();
		}

		private void btnQueryData_Click(object sender, EventArgs e)
		{
			GetData();
		}

		private void TxClientForm_Load(object sender, EventArgs e)
		{
			GetData();
		}
	}
}
