﻿using System;
using System.ServiceModel;

namespace TxServiceApp
{
	[ServiceContract(SessionMode = SessionMode.Required)]
	interface ITxService
	{
		[OperationContract]
		void TxMethod();

		[OperationContract]
		void NonTxMethod();
	}

	[ServiceBehavior(InstanceContextMode = InstanceContextMode.PerSession, 
			ReleaseServiceInstanceOnTransactionComplete = true)]
	class TxPerSessionService : ITxService, IDisposable
	{
		static int _ObjectCount;
		private int _id;

		public TxPerSessionService()
		{
			_id = System.Threading.Interlocked.Increment(ref _ObjectCount);
			Console.WriteLine("+++ Object Created. Id={0}", _id);
		}

		void IDisposable.Dispose()
		{
			Console.WriteLine("--- Object Disposed. Id={0}", _id);
		}

		[OperationBehavior(TransactionScopeRequired = true)]
		public void TxMethod()
		{
			Console.WriteLine("TxMethod() invoked. Id={0}", _id);
		}

		[OperationBehavior(TransactionScopeRequired = false)]
		public void NonTxMethod()
		{
			Console.WriteLine("NonTxMethod() invoked. Id={0}", _id);
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Transactional Service Instance Mode Test Sample......");

			ServiceHost host = new ServiceHost(typeof(TxPerSessionService));
			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
