﻿using System;
using System.Collections.Generic;
using System.Text;
using MsmqClientApp.MsmqService;
using System.ServiceModel;

namespace MsmqClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Exactly-Once Test Client Sample......");

			// ExactlyOnce 테스트를 위해 configuration에 설정된 바인딩 설정을 변경하여
			// 다양한 상황을 테스트 한다.
			string exactlyOnceConfig = "NetMsmqBinding_IMsmqService";
			string nonExactlyOnceConfig = "NetMsmqBinding_IMsmqService1";
			MsmqServiceClient proxy;
			NetMsmqBinding binding;

			// ExactlyOnce = true 를 사용하는 바인딩을 사용하여 트랜잭션 큐에 메시지 전송
			// (호출 성공)
			proxy = new MsmqServiceClient(exactlyOnceConfig);
			binding = (NetMsmqBinding)proxy.Endpoint.Binding;
			binding.ExactlyOnce = true;
			try {
				proxy.TestMethod("#1 Call transactional queue with ExactlyOnce = true");
				proxy.TestMethod("#2 Call transactional queue with ExactlyOnce = true");
				proxy.TestMethod("#3 Call transactional queue with ExactlyOnce = true");
			}
			finally {
				(proxy as IDisposable).Dispose();
			}
			Console.WriteLine("Call the service using transactional queue with ExactlyOnce = true");

			// ExactlyOnce = false를 사용하는 바인딩을 사용하여 트랜잭션 큐에 메시지 전송
			// 클라이언트는 오류가 없으나 배달은 성공하지 못하며 배달 못한 메시지 큐에
			// "비트랜잭션 메시지" 클래스의 메시지가 삽입됨.
			proxy = new MsmqServiceClient(exactlyOnceConfig);
			binding = (NetMsmqBinding)proxy.Endpoint.Binding;
			binding.ExactlyOnce = false;
			try {
				proxy.TestMethod("Call transactional queue with ExactlyOnce = false");
			}
			finally {
				(proxy as IDisposable).Dispose();
			}
			Console.WriteLine("Call the service using transactional queue with ExactlyOnce = false");

			// ExactlyOnce = true 를 사용하는 바인딩을 사용하여 비 트랜잭션 큐에 메시지 전송
			// 클라이언트는 오류가 없으나 배달은 성공하지 못하며 배달 못한 트랜잭션 메시지 큐에
			// "비트랜잭션 메시지" 클래스의 메시지가 삽입됨.
			proxy = new MsmqServiceClient(nonExactlyOnceConfig);
			binding = (NetMsmqBinding)proxy.Endpoint.Binding;
			binding.ExactlyOnce = true;
			try {
				proxy.TestMethod("Call non-transactional queue with ExactlyOnce = true");
			}
			finally {
				(proxy as IDisposable).Dispose();
			}
			Console.WriteLine("Call the service using non-transactional queue with ExactlyOnce = true");

			// ExactlyOnce = false를 사용하는 바인딩을 사용하여 비 트랜잭션 큐에 메시지 전송
			proxy = new MsmqServiceClient(nonExactlyOnceConfig);
			binding = (NetMsmqBinding)proxy.Endpoint.Binding;
			binding.ExactlyOnce = false;
			try {
				proxy.TestMethod("Call non-transactional queue with ExactlyOnce = false");
			}
			finally {
				(proxy as IDisposable).Dispose();
			}
			Console.WriteLine("Call the service using non-transactional queue with ExactlyOnce = false");

		}
	}
}
