﻿using System;
using System.ServiceModel;
using System.Messaging;
using System.ServiceModel.Description;

namespace MsmqServiceApp
{
	[ServiceContract]
	interface IMsmqService
	{
		[OperationContract(IsOneWay = true)]
		void TestMethod(string arg);
	}

	class MsmqService : IMsmqService
	{
		public void TestMethod(string arg)
		{
			Console.WriteLine("TestMethod invoked... arg={0}", arg);
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Exactly-Once-Delivery Test Application");

			// 수신 큐 생성
			string txQueuePath = ".\\private$\\GenericTxQueue";
			if (MessageQueue.Exists(txQueuePath) == false) {
				MessageQueue.Create(txQueuePath, true);			// 트랜잭션 큐 생성
			}
			string nonTxQueuePath = ".\\private$\\GenericQueue";
			if (MessageQueue.Exists(nonTxQueuePath) == false) {
				MessageQueue.Create(nonTxQueuePath, false);		// 비 트랜잭션 큐 생성
			}

			// Exactly-Once 를 사용하는 바인딩 설정
			// ExactlyOnce 속성이 true 이므로 트랜잭션 큐가 반드시 사용되어야 한다.
			NetMsmqBinding binding1 = new NetMsmqBinding();
			binding1.ExactlyOnce = true;
			binding1.Security.Mode = NetMsmqSecurityMode.None;
			// Exactly-Once 를 사용하지 않는 바인딩 설정
			// ExactlyOnce 속성이 false 이므로 트랜잭션 큐가 반드시 사용되면 안된다.
			NetMsmqBinding binding2 = new NetMsmqBinding();
			binding2.ExactlyOnce = false;
			binding2.Security.Mode = NetMsmqSecurityMode.None;
			// 서비스 호스트 및 종점 설정
			ServiceHost host = new ServiceHost(typeof(MsmqService),
				new Uri("http://localhost/wcf/example/msmq/exactlyonce"));	// only for Metadata
			// 트랜잭션 큐에 ExactlyOnce 속성을 false 로 지정한 바인딩이 사용되거나
			// 비 트랜잭션 큐에 ExactlyOnce 속성을 true로 지정한 바인딩이 사용되면
			// InvalidOperationException 이 ServiceHost.Open 에서 발생한다.
			host.AddServiceEndpoint(typeof(IMsmqService), binding1,
				"net.msmq://localhost/private/GenericTxQueue");
			host.AddServiceEndpoint(typeof(IMsmqService), binding2,
				"net.msmq://localhost/private/GenericQueue");
			// 메타데이터 HTTP-Get 설정
			ServiceMetadataBehavior behavior = host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			if (behavior == null) {
				behavior = new ServiceMetadataBehavior();
				host.Description.Behaviors.Add(behavior);
			}
			behavior.HttpGetEnabled = true;
			// 서비스 구동
			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
