﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace SessionfulServiceApp
{
	[ServiceContract(SessionMode = SessionMode.Required)]
	[DeliveryRequirements(QueuedDeliveryRequirements = QueuedDeliveryRequirementsMode.Required)]
	interface IMsmqSessionfulService
	{
		[OperationContract(IsOneWay = true)]
		void TestMethod1();

		[OperationContract(IsOneWay = true)]
		void TestMethod2();

		[OperationContract(IsOneWay = true, IsInitiating = false, IsTerminating = true)]
		void TestMethod3();
	}

	class MsmqSessionfulService : IMsmqSessionfulService
	{
		[OperationBehavior(TransactionScopeRequired = true, TransactionAutoComplete = false)]
		public void TestMethod1()
		{
			Console.WriteLine("TestMethod1() invoked...");
			Console.WriteLine("  Session Id = {0}", OperationContext.Current.SessionId);
		}

		[OperationBehavior(TransactionScopeRequired = true, TransactionAutoComplete = false)]
		public void TestMethod2()
		{
			Console.WriteLine("TestMethod2() invoked...");
			Console.WriteLine("  Session Id = {0}", OperationContext.Current.SessionId);
		}

		[OperationBehavior(TransactionScopeRequired = true, TransactionAutoComplete = true)]
		public void TestMethod3()
		{
			Console.WriteLine("TestMethod3() invoked...");
			Console.WriteLine("  Session Id = {0}", OperationContext.Current.SessionId);
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Sessionful MSMQ Service Sample......");

			NetMsmqBinding binding = new NetMsmqBinding();
			binding.Security.Mode = NetMsmqSecurityMode.None;

			ServiceHost host = new ServiceHost(typeof(MsmqSessionfulService),
				new Uri("http://localhost/wcf/example/msmq/session"),
				new Uri("net.msmq://localhost/private/generictxqueue"));

			host.AddServiceEndpoint(typeof(IMsmqSessionfulService), binding, "");

			ServiceMetadataBehavior behavior = host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			if (behavior == null) {
				behavior = new ServiceMetadataBehavior();
				host.Description.Behaviors.Add(behavior);
			}
			behavior.HttpGetEnabled = true;

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
