﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace TxServiceApp
{
	public partial class TxServiceForm : Form
	{
		private ServiceHost _host;

		public TxServiceForm()
		{
			InitializeComponent();
			MsmqTxService.UIForm = this;

			btnOpen.Enabled = true;
			btnCloseService.Enabled = false;
		}

		public bool AbortTransaction
		{
			get { return ckbAbortTransaction.Checked; }
		}

		public void WriteServiceLog(string logMessage)
		{
			MethodInvoker func = delegate() {
				int index = lbServiceLog.Items.Add(logMessage);
				lbServiceLog.SelectedIndex = index;
			};

			if (this.InvokeRequired == true) {
				this.Invoke(func);
			}
			else {
				func();
			}
		}

		private void btnOpen_Click(object sender, EventArgs e)
		{
			_host = new ServiceHost(typeof(MsmqTxService),
				new Uri("http://localhost/wcf/example/msmq/tx"),
				new Uri("net.msmq://localhost/private/generictxqueue"));

			NetMsmqBinding binding = new NetMsmqBinding();
			binding.Security.Mode = NetMsmqSecurityMode.None;

			_host.AddServiceEndpoint(typeof(IMsmqTxService), binding, "");

			ServiceMetadataBehavior behavior = _host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			if (behavior == null) {
				behavior = new ServiceMetadataBehavior();
				_host.Description.Behaviors.Add(behavior);
			}
			behavior.HttpGetEnabled = true;

			_host.Open();

			btnOpen.Enabled = false;
			btnCloseService.Enabled = true;
		}

		private void btnCloseService_Click(object sender, EventArgs e)
		{
			if (_host != null) {
				_host.Close();
			}
			_host = null;
			btnOpen.Enabled = true;
			btnCloseService.Enabled = false;
		}

		private void TxServiceForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (_host != null) {
				_host.Close();
			}
		}

		private void lnkClearLog_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			lbServiceLog.Items.Clear();
		}
	}

	[ServiceContract]
	[DeliveryRequirements(QueuedDeliveryRequirements = QueuedDeliveryRequirementsMode.Required)]
	interface IMsmqTxService
	{
		[OperationContract(IsOneWay = true)]
		void TxServiceMethod(string data);
	}

	class MsmqTxService : IMsmqTxService
	{
		public static TxServiceForm UIForm { get; set; }

		[OperationBehavior(TransactionScopeRequired = true)]
		public void TxServiceMethod(string data)
		{
			string logMessage = String.Format("[{0:HH:mm:ss.sss}] Method Invoke: data = {1}.", 
				DateTime.Now, data);
			UIForm.WriteServiceLog(logMessage);
			if (UIForm.AbortTransaction == true) {
				UIForm.WriteServiceLog("    Transaction will be aborted.");
				// 예외를 발생 시킴으로써 트랜잭션을 강제로 취소 한다.
				throw new Exception("Transaction is aborted by user.");
			}
			UIForm.WriteServiceLog("    Transaction will be completed.");
		}
	}
}
