﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Security.Principal;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel.Security;
using System.IdentityModel.Selectors;
using System.Net.Security;

namespace SecureServiceApp
{
	[ServiceContract]
	interface ISecureService
	{
		[OperationContract(IsOneWay = true)]
		void SecureMethod(string arg);
	}

	class SecureService : ISecureService
	{
		public void SecureMethod(string arg)
		{
			Console.WriteLine("SecureMethod invoked... arg={0}", arg);
			ServiceSecurityContext ctx = ServiceSecurityContext.Current;
			if (ctx == null || ctx.IsAnonymous) {
				Console.WriteLine("  Anonymous client");
			}
			else {
				Console.WriteLine("  User = {0}", ServiceSecurityContext.Current.PrimaryIdentity.Name);
			}
			WindowsIdentity identity = WindowsIdentity.GetCurrent();
			Console.WriteLine("  Windows Account = {0}", identity.Name);
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("MSMQ Security Test Service......");

			NetMsmqBinding binding = new NetMsmqBinding();
			binding.Security.Mode = NetMsmqSecurityMode.Message;
			binding.Security.Message.ClientCredentialType = MessageCredentialType.UserName;

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("http://localhost/wcf/example/security/msmq"),
				new Uri("net.msmq://localhost/private/generictxqueue"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");
			host.Description.Behaviors.Add(behavior);
			host.Credentials.ServiceCertificate.SetCertificate(
				"CN=localhost", StoreLocation.LocalMachine, StoreName.My);

			host.Credentials.UserNameAuthentication.UserNamePasswordValidationMode =
				UserNamePasswordValidationMode.Custom;
			host.Credentials.UserNameAuthentication.CustomUserNamePasswordValidator =
				new CustomUserNameValidator();

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}

	class CustomUserNameValidator : UserNamePasswordValidator
	{
		public override void Validate(string userName, string password)
		{
			if (userName != "TempUser" || password != "1234") {
				throw new System.IdentityModel.Tokens.SecurityTokenException("Unknown User/Password");
			}
		}
	}
}
