﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.Net.Security;
using System.ServiceModel.Description;
using System.Security.Cryptography.X509Certificates;

namespace SecureServiceApp
{
	#region "보안을 사용하지 않는 서비스 구현"

	[ServiceContract]
	interface ISecureService1
	{
		[OperationContract]
		void SecureMethod1(string arg);
	}

	class SecureService1 : ISecureService1
	{
		public void SecureMethod1(string arg)
		{
			Console.WriteLine("SecureMethod1 invoked : arg={0}", arg);
		}

		public static ServiceHost CreateServiceHost()
		{
			//BasicHttpBinding binding = new BasicHttpBinding();
			//binding.Security.Mode = BasicHttpSecurityMode.None;

			ServiceHost host = new ServiceHost(typeof(SecureService1),
				new Uri("http://localhost/wcf/example/security/test/service1"));
			//host.AddServiceEndpoint(typeof(ISecureService1), binding, "");

			//ServiceMetadataBehavior behavior = host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			//if (behavior == null) {
			//    behavior = new ServiceMetadataBehavior();
			//    host.Description.Behaviors.Add(behavior);
			//}
			//behavior.HttpGetEnabled = true;

			return host;
		}
	}
	#endregion

	#region "트랜스포트 보안(익명 인증)을 사용하는 서비스 구현"

	[ServiceContract]
	interface ISecureService2
	{
		[OperationContract]
		void SecureMethod2(string arg);
	}

	class SecureService2 : ISecureService2
	{
		public void SecureMethod2(string arg)
		{
			Console.WriteLine("SecureMethod2 invoked : arg={0}", arg);
		}

		public static ServiceHost CreateServiceHost()
		{
			//BasicHttpBinding binding = new BasicHttpBinding();
			//binding.Security.Mode = BasicHttpSecurityMode.Transport;
			//binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.None;

			ServiceHost host = new ServiceHost(typeof(SecureService2),
				new Uri("https://localhost/wcf/example/security/test/service2"));
			//host.AddServiceEndpoint(typeof(ISecureService2), binding, "");

			//ServiceMetadataBehavior behavior = host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			//if (behavior == null) {
			//    behavior = new ServiceMetadataBehavior();
			//    host.Description.Behaviors.Add(behavior);
			//}
			//behavior.HttpsGetEnabled = true;

			return host;
		}
	}
	#endregion

	#region "트랜스포트 보안(Windows 인증)을 사용하는 서비스 구현"

	[ServiceContract]
	interface ISecureService3
	{
		[OperationContract]
		void SecureMethod3(string arg);
	}

	class SecureService3 : ISecureService3
	{
		public void SecureMethod3(string arg)
		{
			Console.WriteLine("SecureMethod3 invoked : arg={0}", arg);
		}

		public static ServiceHost CreateServiceHost()
		{
			//BasicHttpBinding binding = new BasicHttpBinding();
			//binding.Security.Mode = BasicHttpSecurityMode.Transport;
			//binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Windows;

			ServiceHost host = new ServiceHost(typeof(SecureService3),
				new Uri("https://localhost/wcf/example/security/test/service3"));
			//host.AddServiceEndpoint(typeof(ISecureService3), binding, "");

			//ServiceMetadataBehavior behavior = host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			//if (behavior == null) {
			//    behavior = new ServiceMetadataBehavior();
			//    host.Description.Behaviors.Add(behavior);
			//}
			//behavior.HttpsGetEnabled = true;

			return host;
		}
	}
	#endregion

	#region "서명만 사용하는 메시지 수준의 보안"

	[ServiceContract(ProtectionLevel = ProtectionLevel.Sign)]
	interface ISecureService4
	{
		[OperationContract]
		void SecureMethod4(string arg);
	}

	class SecureService4 : ISecureService4
	{
		public void SecureMethod4(string arg)
		{
			Console.WriteLine("SecureMethod4 invoked : arg={0}", arg);
		}

		public static ServiceHost CreateServiceHost()
		{
			//BasicHttpBinding binding = new BasicHttpBinding();
			//binding.Security.Mode = BasicHttpSecurityMode.Message;
			//binding.Security.Message.ClientCredentialType = BasicHttpMessageCredentialType.Certificate;

			ServiceHost host = new ServiceHost(typeof(SecureService4),
				new Uri("http://localhost/wcf/example/security/test/service4"));
			//host.AddServiceEndpoint(typeof(ISecureService4), binding, "");

			//// 서비스 인증서 설정
			//host.Credentials.ServiceCertificate.SetCertificate(
			//    "CN=localhost", StoreLocation.LocalMachine, StoreName.My);

			//ServiceMetadataBehavior behavior = host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			//if (behavior == null) {
			//    behavior = new ServiceMetadataBehavior();
			//    host.Description.Behaviors.Add(behavior);
			//}
			//behavior.HttpGetEnabled = true;

			return host;
		}
	}
	#endregion

	#region "서명과 암호화만 사용하는 메시지 수준의 보안"

	[ServiceContract(ProtectionLevel = ProtectionLevel.EncryptAndSign)]
	interface ISecureService5
	{
		[OperationContract]
		void SecureMethod5(string arg);
	}

	class SecureService5 : ISecureService5
	{
		public void SecureMethod5(string arg)
		{
			Console.WriteLine("SecureMethod5 invoked : arg={0}", arg);
		}

		public static ServiceHost CreateServiceHost()
		{
			//BasicHttpBinding binding = new BasicHttpBinding();
			//binding.Security.Mode = BasicHttpSecurityMode.Message;
			//binding.Security.Message.ClientCredentialType = BasicHttpMessageCredentialType.Certificate;

			ServiceHost host = new ServiceHost(typeof(SecureService5),
				new Uri("http://localhost/wcf/example/security/test/service5"));
			//host.AddServiceEndpoint(typeof(ISecureService5), binding, "");

			//// 서비스 인증서 설정
			//host.Credentials.ServiceCertificate.SetCertificate(
			//    "CN=localhost", StoreLocation.LocalMachine, StoreName.My);

			//ServiceMetadataBehavior behavior = host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			//if (behavior == null) {
			//    behavior = new ServiceMetadataBehavior();
			//    host.Description.Behavi ors.Add(behavior);
			//}
			//behavior.HttpGetEnabled = true;

			return host;
		}
	}
	#endregion

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Simple Secure Service Sample......");

			ServiceHost host1 = SecureService1.CreateServiceHost();
			ServiceHost host2 = SecureService2.CreateServiceHost();
			ServiceHost host3 = SecureService3.CreateServiceHost();
			ServiceHost host4 = SecureService4.CreateServiceHost();
			ServiceHost host5 = SecureService5.CreateServiceHost();

			host1.Open();
			Console.WriteLine("Non-secure service started...");
			host2.Open();
			Console.WriteLine("Secure service with transport security with anonymous auth. started...");
			host3.Open();
			Console.WriteLine("Secure service with transport security with windows auth. started...");
			host4.Open();
			Console.WriteLine("Secure service with message security(sign-only) started...");
			host5.Open();
			Console.WriteLine("Secure service with message security(encrypt and sign) started...");

			Console.WriteLine("Press any key to stop the services...");
			Console.ReadKey(true);

			host1.Close();
			host2.Close();
			host3.Close();
			host4.Close();
			host5.Close();
		}
	}
}
