﻿using System;
using System.Collections.Specialized;
using System.IdentityModel.Selectors;
using System.IdentityModel.Tokens;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Security;
using System.Web.Security;
using System.Security.AccessControl;
using System.Security.Principal;

namespace SecureServiceApp
{
	[ServiceContract]
	interface ISecureService
	{
		[OperationContract]
		void SecureMethod(string arg);
	}

	class SecureService : ISecureService
	{
		public void SecureMethod(string arg)
		{
			Console.WriteLine("SecureMethod invoked... arg={0}", arg);
			ServiceSecurityContext ctx = OperationContext.Current.ServiceSecurityContext;
			if (ctx.IsAnonymous) {
				Console.WriteLine("  Anonymous client");
			}
			else {
				Console.WriteLine("  User = {0}", ctx.PrimaryIdentity.Name);
				Console.WriteLine("  AuthenticateType = {0}", ctx.PrimaryIdentity.AuthenticationType);
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Username/Password Authentication Test Service......");

			WSHttpBinding binding = new WSHttpBinding();
			binding.Security.Mode = SecurityMode.Message;
			binding.Security.Message.ClientCredentialType = MessageCredentialType.UserName;

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("http://localhost/wcf/example/security/usernameauth"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");
			host.Description.Behaviors.Add(behavior);
			host.Credentials.ServiceCertificate.SetCertificate(
				"CN=localhost", StoreLocation.LocalMachine, StoreName.My);
			//host.Credentials.UserNameAuthentication.UserNamePasswordValidationMode =
			//    UserNamePasswordValidationMode.MembershipProvider;
			//host.Credentials.UserNameAuthentication.MembershipProvider = 
			//    Membership.Providers["sqlProvider"];

			//host.Credentials.UserNameAuthentication.UserNamePasswordValidationMode =
			//    UserNamePasswordValidationMode.Custom;
			//CustomUserNameValidator validator = new CustomUserNameValidator();
			//host.Credentials.UserNameAuthentication.CustomUserNamePasswordValidator = validator;

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}

	class CustomUserNameValidator : UserNamePasswordValidator
	{
		public override void Validate(string userName, string password)
		{
			// check the username and password.
			if (userName != "TempUser" || password != "1234") {
				throw new SecurityTokenException("Unknown user.");
			}
			Console.WriteLine("{0} is a valid user.", userName);
		}
	}
}
