﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Configuration;
using System.Configuration;

namespace BindingExtension
{
	public class CustomHttpBindingConfigElement : StandardBindingElement
	{
		private ConfigurationPropertyCollection _properties;

		public CustomHttpBindingConfigElement() : this(null)
		{
		}

		public CustomHttpBindingConfigElement(string configName)
			: base(configName)
		{
		}

		[ConfigurationProperty("usernameSecurityEnabled", DefaultValue = false)]
		public bool UsernameSecurityEnabled
		{
			get { return (bool)base["usernameSecurityEnabled"]; }
			set { base["usernameSecurityEnabled"] = value; }
		}

		[ConfigurationProperty("transferMode", DefaultValue = false)]
		public TransferMode TransferMode
		{
			get { return (TransferMode)base["transferMode"]; }
			set { base["transferMode"] = value; }
		}

		protected override Type BindingElementType
		{
			get { return typeof(CustomHttpBinding); }
		}

		protected override ConfigurationPropertyCollection Properties
		{
			get
			{
				if (_properties == null) {
					_properties = base.Properties;
					_properties.Add(new ConfigurationProperty("usernameSecurityEnabled", typeof(bool)));
					_properties.Add(new ConfigurationProperty("transferMode", typeof(TransferMode)));
				}
				return _properties;
			}
		}

		protected override void OnApplyConfiguration(Binding binding)
		{
			if (binding == null) {
				throw new ArgumentNullException("binding");
			}
			if (binding.GetType() != typeof(CustomHttpBinding)) {
				throw new ArgumentException("Invalid binding type");
			}
			CustomHttpBinding customHttpbinding = (CustomHttpBinding)binding;
			customHttpbinding.UsernameSecurityEnabled = this.UsernameSecurityEnabled;
			customHttpbinding.TransferMode = this.TransferMode;
		}

		protected override void InitializeFrom(Binding binding)
		{
			base.InitializeFrom(binding);

			CustomHttpBinding customHttpBinding = (CustomHttpBinding)binding;
			this.UsernameSecurityEnabled = customHttpBinding.UsernameSecurityEnabled;
			this.TransferMode = customHttpBinding.TransferMode;
		}
	}

	public class CustomHttpBindingCollectionElement : StandardBindingCollectionElement<
		CustomHttpBinding, CustomHttpBindingConfigElement>
	{
		internal static CustomHttpBindingConfigElement GetBindingSection(string configName)
		{
			BindingsSection bindingsSection = (BindingsSection)ConfigurationManager.GetSection(
				"system.serviceModel/bindings");
			CustomHttpBindingCollectionElement customBindingSection = 
				(CustomHttpBindingCollectionElement)bindingsSection["customHttpBinding"];
			CustomHttpBindingConfigElement bindingConfig = 
				(CustomHttpBindingConfigElement)(customBindingSection.Bindings[configName]);
			return bindingConfig;
		}
	}
}
