﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Runtime.Serialization;
using System.IO;
using System.Xml;
using System.Text;

namespace MessageObjectSample
{
	[DataContract(Namespace="http://temp.org")]
	class BookInfo
	{
		[DataMember]
		public string Title;
		[DataMember]
		public string Author;
	}

	class Program
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Message Manupluation Sample......");

			CreateSimpleMessage();

			Message message1 = CreateMessageWithDataContract();

			// 헤더 추가
			MessageHeader myHeader = MessageHeader.CreateHeader("MyHeader", "urn:Test", "Header Info");
			message1.Headers.Add(myHeader);

			Console.WriteLine("--- Message Serialize/Deserialize ----------------------------------");

			MemoryStream stream = new MemoryStream();
			SerializeMessage(message1, stream);
			stream.Position = 0;
			Message message2 = DeserializeMessage(stream);
			Console.WriteLine(message2);
			BookInfo book = message2.GetBody<BookInfo>();
			Console.WriteLine("Title={0}  Author={1}", book.Title, book.Author);
		}

		static void CreateSimpleMessage()
		{
			Console.WriteLine("--- Simple Message Creation ----------------------------------------");
			Message message = Message.CreateMessage(
				MessageVersion.Soap11, "urn:MyAction", "Hello");
			Console.WriteLine(message.ToString());
		}

		static Message CreateMessageWithDataContract()
		{
			Console.WriteLine("--- DataContract based Message Creation ----------------------------");
			BookInfo book = new BookInfo();
			book.Title = "WCF";
			book.Author = "A.Writer";

			Message message = Message.CreateMessage(
				MessageVersion.Soap11, "urn:MyAction", book);
			Console.WriteLine(message.ToString());
			return message;
		}

		static void SerializeMessage(Message message, Stream stream)
		{
			XmlDictionaryWriter writer =
				XmlDictionaryWriter.CreateTextWriter(stream, Encoding.UTF8, false);
			// 메시지 전체를 직렬화 하는 예제
			message.WriteMessage(writer);
			writer.Close();
		}

		static Message DeserializeMessage(Stream stream)
		{
			XmlDictionaryReader reader =
				XmlDictionaryReader.CreateTextReader(stream, XmlDictionaryReaderQuotas.Max);
			Message message = Message.CreateMessage(reader, 8192, MessageVersion.Soap11);
			return message;
		}
	}
}
