﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.Serialization;

namespace GenericsDataContractService
{
	[DataContract(Namespace = "http://noname.com/generics/data")]
	public class SomeTypeA<T>
	{
		[DataMember]
		public T Member = default(T);
	}

	[DataContract(Name="SomeTypeB_{0}", Namespace = "http://noname.com/generics/data")]
	public class SomeTypeB<T>
	{
		[DataMember]
		public T Member = default(T);
	}

	[DataContract(Name = "SomeTypeC", Namespace = "http://noname.com/generics/data")]
	public class SomeTypeC<T>
	{
		[DataMember]
		public T Member = default(T);
	}

	[DataContract(Namespace = "http://noname.com/generics/data")]
	public struct CustomType
	{
		[DataMember]
		public int a;

		public CustomType(int val)
		{ a = val; }
	}

	[ServiceContract(Namespace = "http://noname.com/generics")]
	interface IGenericsTest
	{
		[OperationContract]
		int Operation1(SomeTypeA<int> arg);
		[OperationContract]
		DateTime Operation2(SomeTypeA<DateTime> arg);
		[OperationContract]
		CustomType Operation3(SomeTypeA<CustomType> arg);
		[OperationContract]
		int Operation4(SomeTypeB<int> arg);
		[OperationContract]
		DateTime Operation5(SomeTypeB<DateTime> arg);
		[OperationContract]
		CustomType Operation6(SomeTypeB<CustomType> arg);
		[OperationContract]
		CustomType Operation7(SomeTypeC<CustomType> arg);
		// 다음과 같은 선언은 동일한 이름의 데이터 계약이 존재한다는
		// InvalidOperationException 예외를 유발한다.
		//[OperationContract]
		//String Operation8(SomeTypeC<String> arg);
	}

	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class TestService : IGenericsTest
	{
		public int Operation1(SomeTypeA<int> arg)
		{ return arg.Member; }

		public DateTime Operation2(SomeTypeA<DateTime> arg)
		{ return arg.Member; }

		public CustomType Operation3(SomeTypeA<CustomType> arg)
		{ return arg.Member; }

		public int Operation4(SomeTypeB<int> arg)
		{ return arg.Member; }

		public DateTime Operation5(SomeTypeB<DateTime> arg)
		{ return arg.Member; }

		public CustomType Operation6(SomeTypeB<CustomType> arg)
		{ return arg.Member; }

		public CustomType Operation7(SomeTypeC<CustomType> arg)
		{ return arg.Member; }

		public String Operation8(SomeTypeC<String> arg)
		{ return arg.Member; }
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Generics Data Contract Service Example...");

			ServiceHost host = new ServiceHost(
				typeof(TestService),
				new Uri("http://localhost/wcf/example/testservice"));

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IGenericsTest),
				new BasicHttpBinding(),
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
