﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace MessageContractService
{
	// 서비스 계약 정의
	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IBookStore
	{
		[OperationContract]
		OrderResponse Order(OrderRequest orderRequest);
	}

	// SOAP Request 관련 메시지 계약
	[MessageContract(WrapperName = "Order")]
	class OrderRequest
	{
		[MessageHeader]
		public string Password = null;
		[MessageBodyMember(Name = "userid", Order = 1)]
		public string Userid = null;
		[MessageBodyMember(Name = "isbn", Order = 2)]
		public string Isbn = null;
		[MessageBodyMember(Name = "amount", Order = 3)]
		public int Amount = 0;
	}

	// SOAP Response 관련 메시지 계약
	[MessageContract]
	class OrderResponse
	{
		[MessageBodyMember]
		public int OrderResult = 0;
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class BookStoreService : IBookStore
	{
		private static int id = 0;

		private int GetUnitPrice(string isbn)
		{
			return 35000;
		}

		// 도서 주문
		public OrderResponse Order(OrderRequest orderRequest)
		{
			string userid = orderRequest.Userid;
			string isbn = orderRequest.Isbn;
			int amount = orderRequest.Amount;

			if (String.IsNullOrEmpty(orderRequest.Password) || orderRequest.Password != "password") {
				throw new InvalidOperationException("Invalid Client Call");
			}
			int total = GetUnitPrice(isbn) * amount;
			id += 1;
			// 주문 처리 로직
			// (단순 출력만... -_-; )
			Console.WriteLine(
				"Book Order: orderid={0}\n   userid={1}, isbn={2}, amount={3}, total={4}",
				id, userid, isbn, amount, total);
			// 주문 ID 반환
			OrderResponse response = new OrderResponse();
			response.OrderResult = id;
			return response;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Message Contract Service Example...");

			ServiceHost host = new ServiceHost(
				typeof(BookStoreService),
				new Uri("http://localhost/wcf/example/bookstoreservice"));

			BasicHttpBinding binding = new BasicHttpBinding();

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IBookStore),
				binding,
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
