﻿using System;
using System.ServiceModel;

namespace MaxConnectionSettingClient
{
	// 수동으로 선언한 서비스 계약
	[ServiceContract(Namespace = "http://noname.com/myservice")]
	interface IMyContract
	{
		[OperationContract]
		string SayHello();
	}

	// 수동으로 직접 작성한 프록시 클래스
	class MyContractClient : ClientBase<IMyContract>
	{
		public string SayHello()
		{
			return this.Channel.SayHello();
		}
	}

	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Setting MaxConnections of NetTcpBinding Sample Client......");

			InvokeServiceMultiple();

			Console.WriteLine("Press Any Key to Invoke the Service Again");
			Console.ReadKey(true);
			InvokeServiceMultiple();

			Console.WriteLine("Press Any Key to Terminate Client");
			Console.ReadKey(true);
		}

		// 여러개의 Proxy를 생성하여 서비스를 호출하는 Helper 메쏘드
		private static void InvokeServiceMultiple()
		{
			MyContractClient[] proxies = new MyContractClient[3];

			// 프록시 생성
			for (int i = 0; i < proxies.Length; i++) {
				proxies[i] = new MyContractClient();
			}

			// 서비스 호출
			for (int i = 0; i < proxies.Length; i++) {
				Console.Write("#{0} Test : try to open...", i);
				proxies[i].Open();			// TCP 연결이 여기서 수행된다.
				Console.WriteLine("connection done.");
				Console.WriteLine("   Result = {0}", proxies[i].SayHello());
			}
			Console.WriteLine("Press Any Key to close proxies");
			Console.ReadKey(true);

			// 프록시 종료(dispose)
			// 프록시가 종료되면 TCP 연결이 종료되는 것이 일반적이지만
			// NetTcpBinding의 MaxConnections 속성에 의해 TCP 연결은 곧바로
			// 끊기지 않고 연결 풀(connection pool)에 들어가게 된다.
			for (int i = 0; i < proxies.Length; i++) {
				Console.WriteLine("#{0} Disposing...", i);
				(proxies[i] as IDisposable).Dispose();
			}
		}
	}
}
