﻿using System;
using System.ServiceModel;
using System.Runtime.Serialization;
using System.IO;
using System.Xml;

namespace ReaderQuatasTest
{
	[DataContract]
	class TestObject1
	{
		[DataMember]
		public string StringValue;
	}

	[DataContract]
	class TestObject2
	{
		[DataMember]
		public byte[] BinaryValue;
	}

	[DataContract]
	class TestObject2_1
	{
		[DataMember]
		public long LongValue;
	}

	[DataContract]
	class TestObject3
	{
		[DataMember]
		public TestObject3 Ref;
	}

	class Program
	{
		static void Main(string[] args)
		{
			TestObject1 obj1 = new TestObject1();
			obj1.StringValue = new String('X', 8);		// 8 글자의 문자열

			TestObject2 obj2 = new TestObject2();
			obj2.BinaryValue = new byte[8];				// 8 바이트 버퍼

			TestObject2_1 obj21 = new TestObject2_1();
			obj21.LongValue = 0xff00ff00;

			TestObject3 obj3 = new TestObject3();		// 객체 depth가 5인 객체 (TestObject3 자체도 고려해야 함)
			TestObject3 obj31 = new TestObject3();
			TestObject3 obj32 = new TestObject3();
			TestObject3 obj33 = new TestObject3();
			obj3.Ref = obj31;
			obj31.Ref = obj32;
			obj32.Ref = obj33;

			TestReaderQuatas(obj1, 4, 8, 32);		// OK - 문자열을 읽으므로 maxStringContentLength 값만이 유효하다
			TestReaderQuatas(obj2, 8, 2, 32);		// OK - 바이트 배열을 읽으므로 maxArrayLength 만이 유효하다.
			TestReaderQuatas(obj21, 1, 1, 32);		// OK - Long 타입을 읽기 위해 바이트 배열을 사용하지 않는다.
			TestReaderQuatas(obj3, 16, 16, 5);		// OK
		}

		static void TestReaderQuatas(object obj, int maxArrayLength, int maxStringContentLength, int maxDepth)
		{
			Console.WriteLine("Testing with maxArrayLength={0}, maxStringContentLength={1}, maxDepth={2}",
				maxArrayLength, maxStringContentLength, maxDepth);

			string filename = obj.GetType().Name + ".xml";
			FileStream stream = new FileStream(filename, FileMode.Create);
			XmlDictionaryWriter writer = XmlDictionaryWriter.CreateTextWriter(stream);
			DataContractSerializer serializer = new DataContractSerializer(obj.GetType());
			serializer.WriteObject(writer, obj);
			writer.Flush();

			XmlDictionaryReaderQuotas readerQuotas = new XmlDictionaryReaderQuotas();
			readerQuotas.MaxArrayLength = maxArrayLength;
			readerQuotas.MaxStringContentLength = maxStringContentLength;
			readerQuotas.MaxDepth = maxDepth;
			stream.Position = 0;

			XmlDictionaryReader reader = XmlDictionaryReader.CreateTextReader(stream, readerQuotas);
			obj = serializer.ReadObject(reader);
			reader.Close();
		}
	}
}
