﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using FileMonitorClientApp.FileMonitorService;
using System.ServiceModel;

namespace FileMonitorClientApp
{
	public partial class ServiceUIForm : Form
	{
		private FileMonitorClient _Proxy;

		public ServiceUIForm()
		{
			InitializeComponent();
		}

		private void ServiceUIForm_Load(object sender, EventArgs e)
		{
			// 프록시 생성 및 서비스 연결
			_Proxy = new FileMonitorClient();
			try {
				_Proxy.Open();
			}
			catch (CommunicationException ex) {
				string message = String.Format(
					"Cannot connect to File Monitor Service. Start the service and restart.\r\n\r\n{0}", ex.Message);
				MessageBox.Show(message, "File Monitor Service Client",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				_Proxy = null;
				this.Close();
				return;
			}
			// 현재 설정된 파일 경로 표시
			string filePath = _Proxy.GetFilePath();
			if (String.IsNullOrEmpty(filePath) == true) {
				txtFilePath.Text = "N/A";
			}
			else {
				txtFilePath.Text = filePath;
			}
			// 타이머 가동
			tmrPoll.Enabled = true;
		}

		// 폼이 닫히면 프록시를 닫는다.
		private void ServiceUIForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (_Proxy != null) {
				_Proxy.Close();
			}
		}

		// 파일 선택 대화상자를 표시한다.
		private void btnFileSelect_Click(object sender, EventArgs e)
		{
			OpenFileDialog dlg = new OpenFileDialog();
			dlg.Title = "Select file to mointor";

			if (dlg.ShowDialog() == DialogResult.OK) {
				txtFilePath.Text = dlg.FileName;
			}
		}

		// 모니터링할 파일을 서비스에 설정한다.
		private void btnSetFilePath_Click(object sender, EventArgs e)
		{
			string filePath = txtFilePath.Text;

			if (System.IO.File.Exists(filePath) == false) {
				MessageBox.Show("Invalid file path or the file is not exists.",
					"File Monitor Service Client",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				txtFilePath.Focus();
				return;
			}
			_Proxy.SetFilePath(filePath);
		}

		// 타이머에 의해 주기적으로 최종 변경된 시간을 표시한다.
		private void tmrPoll_Tick(object sender, EventArgs e)
		{
			DateTime lastModified = _Proxy.GetLastModified();
			if (lastModified == DateTime.MinValue) {
				txtLastModified.Text = "N/A";
			}
			else {
				txtLastModified.Text = lastModified.ToString();
			}
		}

		// 종료 버튼 클릭 핸들러
		private void btnExit_Click(object sender, EventArgs e)
		{
			this.Close();
		}
	}
}