﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace WPFService
{
	public partial class ServiceWindow : System.Windows.Window
	{
		private ObservableCollection<OrderInfo> _Orders;
		private ServiceHost _Host;

		public ServiceWindow()
		{
			InitializeComponent();

			// 데이터 컬렉션 및 바인딩 초기화
			_Orders = new ObservableCollection<OrderInfo>();
			lvwOrders.DataContext = _Orders;
			// 버튼 상태 제어
			btnStart.IsEnabled = true;
			btnStop.IsEnabled = false;
		}

		// 서비스 호스팅 초기화
		private void InitHost()
		{
			// 서비스 호스트 초기화
			_Host = new ServiceHost(typeof(BookStoreService),
				new Uri("http://localhost/wcf/example/bookstoreservice"));
			_Host.AddServiceEndpoint(typeof(IBookStore),
				new WSHttpBinding(), "");
			// 서비스 이벤트 초기화
			_Host.Opening += new EventHandler(ShowServiceState);
			_Host.Opened += new EventHandler(ShowServiceState);
			_Host.Faulted += new EventHandler(ShowServiceState);
			_Host.Closing += new EventHandler(ShowServiceState);
			_Host.Closed += new EventHandler(ShowServiceState);
			// WSDL 활성화
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			_Host.Description.Behaviors.Add(behavior);
			behavior.HttpGetEnabled = true;
		}

		// 서비스 호출 정보를 표시한다.
		internal void ShowOrderInfo(int orderid, string userid, int amount, BookInfo info)
		{
			// OrderInfo 클래스에 필요한 정보를 설정하여 컬렉션에 추가하기만 하면
			// 데이터 바인딩에 의해 자동으로 내용 표시된다.
			OrderInfo order = new OrderInfo(orderid, userid, info.Isbn, amount, info.Title);
			_Orders.Add(order);
		}

		// 서비스 상태를 표시한다.
		private void ShowServiceState(object sender, EventArgs e)
		{
			Color color = Color.FromRgb(128, 128, 128);

			switch (_Host.State) {
			case CommunicationState.Created:
			case CommunicationState.Closed:
				color = Color.FromRgb(128, 0, 0);
				break;
			case CommunicationState.Opening:
			case CommunicationState.Closing:
				color = Color.FromRgb(0, 0, 128);
				break;
			case CommunicationState.Opened:
				color = Color.FromRgb(0, 128, 0);
				break;
			}
			gmCircle.Fill = new SolidColorBrush(color);
			lblServiceState.Text = "Service State: " + _Host.State.ToString();
		}

		// Start 버튼 클릭 이벤트 핸들러
		private void btnStart_Click(object sender, RoutedEventArgs e)
		{
			InitHost();
			_Host.Open();
			btnStart.IsEnabled = false;
			btnStop.IsEnabled = true;
		}

		// Stop 버튼 클릭 이벤트 핸들러
		private void btnStop_Click(object sender, RoutedEventArgs e)
		{
			if (_Host != null) {
				_Host.Close();
				btnStart.IsEnabled = true;
				btnStop.IsEnabled = false;
			}
		}
	}

	// Order 정보를 나타내는 클래스
	// 데이터 바인딩을 위해 사용한다.
	public class OrderInfo
	{
		private int _OrderId;
		private string _UserId;
		private string _Isbn;
		private int _Amount;
		private string _Title;

		public OrderInfo(int orderID, string userId, string isbn, int amount, string title)
		{
			_OrderId = orderID;
			_UserId = userId;
			_Isbn = isbn;
			_Amount = amount;
			_Title = title;
		}

		#region "Order 정보를 위한 읽기 전용 속성들"

		public int OrderId
		{
			get { return _OrderId; }
		}

		public string UserId
		{
			get { return _UserId; }
		}

		public string Isbn
		{
			get { return _Isbn; }
		}

		public int Amount
		{
			get { return _Amount; }
		}

		public string Title
		{
			get { return _Title; }
		}
		#endregion
	}
}