﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.ServiceModel;
using System.Runtime.Serialization;
using System.ServiceModel.Description;

namespace WinFormService
{
	public partial class ServiceForm : Form, IBookStoreUI
	{
		private ServiceHost _Host;

		public ServiceForm()
		{
			InitializeComponent();

			// 서비스에 대한 UI 인터페이스 설정
			BookStoreService.BookStoreUI = this;

			// 버튼 상태 제어
			btnStart.Enabled = true;
			btnStop.Enabled = false;
		}

		// 서비스 호스팅 초기화
		private void InitHost()
		{
			// 서비스 호스트 초기화
			_Host = new ServiceHost(typeof(BookStoreService),
				new Uri("http://localhost/wcf/example/bookstoreservice"));
			_Host.AddServiceEndpoint(typeof(IBookStore),
				new WSHttpBinding(), "");
			// 서비스 이벤트 초기화
			_Host.Opening += new EventHandler(ShowServiceState);
			_Host.Opened += new EventHandler(ShowServiceState);
			_Host.Faulted += new EventHandler(ShowServiceState);
			_Host.Closing += new EventHandler(ShowServiceState);
			_Host.Closed += new EventHandler(ShowServiceState);
			// WSDL 활성화
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			_Host.Description.Behaviors.Add(behavior);
			behavior.HttpGetEnabled = true;
		}

		// 서비스 호출 정보를 표시한다.
		public void ShowOrderInfo(int orderid, string userid, int amount, BookInfo info)
		{
			ListViewItem item = new ListViewItem(
				new string[] { orderid.ToString(), userid, info.Isbn, info.Title, amount.ToString() }, 0);
			lvwOrders.Items.Add(item);
		}

		// 서비스 상태를 표시한다.
		private void ShowServiceState(object sender, EventArgs e)
		{
			Image stateImage = null;

			switch (_Host.State) {
			case CommunicationState.Created:
			case CommunicationState.Closed:
				stateImage = Properties.Resources.RedSignal;
				break;
			case CommunicationState.Opening:
			case CommunicationState.Closing:
				stateImage = Properties.Resources.YellowSignal;
				break;
			case CommunicationState.Opened:
				stateImage = Properties.Resources.GreenSignal;
				break;
			}
			lblServiceState.Image = stateImage;
			lblServiceState.Text = "Service State: " + _Host.State.ToString();
		}

		// Start 버튼 클릭 이벤트 핸들러
		private void btnStart_Click(object sender, EventArgs e)
		{
			InitHost();
			_Host.Open();
			btnStart.Enabled = false;
			btnStop.Enabled = true;
		}

		// Stop 버튼 클릭 이벤트 핸들러
		private void btnStop_Click(object sender, EventArgs e)
		{
			if (_Host != null) {
				_Host.Close();
				btnStart.Enabled = true;
				btnStop.Enabled = false;
			}
		}

		// 폼 종료 이벤트 핸들러
		private void ServiceForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (_Host != null && _Host.State != CommunicationState.Closed) {
				_Host.Close();
			}
		}
	}
}