﻿using System;
using ExceptionClientApp.BadService;
using System.ServiceModel;

namespace ExceptionClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("WCF Basic Exception Handling Client Example......");

			// WSHttpBinding의 SeucrityMode가 None 이므로 세션을 사용하지 않는다.
			// 따라서 예외가 발생하더라도 proxy를 다시 사용할 수 있다.
			using (BadContractClient proxy = new BadContractClient("WSHttpBinding_IBadContract")) {
				try {
					proxy.ThrowException();
				}
				catch (Exception ex) {
					Console.WriteLine(ex.ToString());
				}
				Console.WriteLine("\nChannelState = {0}\n", proxy.InnerChannel.State);
				try {
					proxy.ThrowNullReferenceException();
				}
				catch (Exception ex) {
					Console.WriteLine(ex.ToString());
				}
			}

			Console.WriteLine("-------------------------------------------------------------");

			// NetTcpBinding은 세션을 사용하므로 서비스가 예외를 유발하여
			// 채널이 폴트 상태가 되면 동일 프록시를 사용하여 서비스를 호출하면
			// CommunicationObjectFaultedException 가 발생한다.
			using (BadContractClient proxy = new BadContractClient("NetTcpBinding_IBadContract")) {
				try {
					proxy.ThrowException();
				}
				catch (Exception ex) {
					Console.WriteLine(ex.ToString());
				}
				Console.WriteLine("\nChannelState = {0}\n", proxy.InnerChannel.State);
				try {
					proxy.ThrowNullReferenceException();
				}
				catch (Exception ex) {
					Console.WriteLine(ex.ToString());
					proxy.Abort();
				}
			}

			Console.WriteLine("-------------------------------------------------------------");

			// NetTcpBinding은 세션을 사용하므로 서비스가 예외를 유발하여
			// 채널이 폴트 상태가 되면 동일 프록시를 사용하여 서비스를 호출하면
			// CommunicationObjectFaultedException 가 발생한다.
			BadContractClient safeProxy = new BadContractClient("NetTcpBinding_IBadContract");
			try {
				try {
					safeProxy.ThrowException();
				}
				catch (Exception ex) {
					Console.WriteLine(ex.ToString());
					safeProxy.Abort();
					safeProxy.Close();
					safeProxy = new BadContractClient("NetTcpBinding_IBadContract");
				}
				Console.WriteLine("\nChannelState = {0}\n", safeProxy.InnerChannel.State);
				try {
					safeProxy.ThrowNullReferenceException();
				}
				catch (Exception ex) {
					Console.WriteLine(ex.ToString());
					safeProxy.Abort();
				}
			}
			finally {
				(safeProxy as IDisposable).Dispose();
			}

		}
	}
}
