﻿using System;
using ExceptionClientApp.BadService;
using System.ServiceModel;

namespace ExceptionClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("FaultException Handling Client Example......");

			using (BadContractClient proxy = new BadContractClient("WSHttpBinding_IBadContract")) {
				try {
					proxy.ThrowSimpleFaultException();
				}
				catch (FaultException ex) {
					Console.WriteLine(ex.ToString());
				}
				Console.WriteLine("\nChannelState = {0}\n", proxy.InnerChannel.State);
				try {
					proxy.ThrowWrappedFaultException();
				}
				catch (FaultException ex) {
					Console.WriteLine(ex.ToString());
				}
				Console.WriteLine("\nChannelState = {0}\n", proxy.InnerChannel.State);
				try {
					proxy.ThrowFaultExceptionOfExceptionDetail();
				}
				catch (FaultException<ExceptionDetail> ex) {
					ExceptionDetail detail = ex.Detail;
					Console.WriteLine("Service FaultException [{0}] : {1}", detail.Type, detail.Message);
					Console.WriteLine("Stack Trace :\n{0}", detail.StackTrace);
				}
			}

			Console.WriteLine("-------------------------------------------------------------");

			// 서비스에서 FaultException 혹은 FaultException<T>를 발생하므로 채널이 폴트 상태로
			// 바뀌는 상황은 발생하지 않는다.
			using (BadContractClient proxy = new BadContractClient("NetTcpBinding_IBadContract")) {
				try {
					proxy.ThrowWrappedFaultException();
				}
				catch (Exception ex) {
					Console.WriteLine(ex.ToString());
				}
				Console.WriteLine("\nChannelState = {0}\n", proxy.InnerChannel.State);
				try {
					proxy.ThrowFaultExceptionOfT();
				}
				catch (Exception ex) {
					Console.WriteLine(ex.ToString());
				}
				Console.WriteLine("\nChannelState = {0}\n", proxy.InnerChannel.State);
				try {
					proxy.ThrowFaultExceptionOfExceptionDetail();
				}
				catch (FaultException<ExceptionDetail> ex) {
					ExceptionDetail detail = ex.Detail;
					Console.WriteLine("Service FaultException [{0}] : {1}", detail.Type, detail.Message);
					Console.WriteLine("Stack Trace :\n{0}", detail.StackTrace);
				}
			}
		}
	}
}
