﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using TestServiceA.SvcB;

namespace TestServiceA
{
	[ServiceContract]
	interface IServiceA
	{
		[OperationContract]
		void TestMethod();
		[OperationContract]
		void ReentrantCall();
	}

	// Single 병행성 모드는 데드락이 발생한다.
	// 오류를 없애려면 Reentrant 병행성 모드를 사용하도록 한다.
	[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single,
					ConcurrencyMode = ConcurrencyMode.Single,
					IncludeExceptionDetailInFaults = true)]
	//[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single,
	//                ConcurrencyMode = ConcurrencyMode.Reentrant,
	//                IncludeExceptionDetailInFaults = true)]
	class ServiceA : IServiceA
	{
		// 클라이언트가 호출하는 메쏘드
		public void TestMethod()
		{
			Console.WriteLine("ServiceA : call from client...");
			// 외부 서비스 호출
			using (ServiceBClient proxy = new ServiceBClient()) {
				proxy.OutgoingCall();
			}
			Console.WriteLine("ServiceA : successful call to ServiceB");
		}

		// 외부 서비스(ServiceB)가 호출하는 메쏘드.
		public void ReentrantCall()
		{
			Console.WriteLine("ServiceA : call from ServiceB");
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Reentrant Concurrency Mode Test : ServiceA ......");
			ServiceHost host = new ServiceHost(typeof(ServiceA),
				new Uri("http://localhost/wcf/example/concurrencymode/reentrant/serviceA"));
			host.Open();
			Console.WriteLine("ServiceA started......");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
