﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

using SessionfulClientApp.SessionfulService;

namespace SessionfulClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Sessionful Client Session Id Test Sample......");

			//NetTcpBindingSessionIdTest();
			//NetNamedPipeBindingSessionIdTest();
			//WSHttpBindingSecureSessionIdTest();
			WSDualHttpBindingReliableSessionIdTest();
		}

		static void NetTcpBindingSessionIdTest()
		{
			Console.WriteLine("\nNetTcpBinding Session Id Test......");

			using (SessionfulClient proxy = 
						new SessionfulClient("NetTcpBinding_ISessionful")) {
				string sessionId = proxy.InnerChannel.SessionId;
				Console.WriteLine("Session Id before Open() : {0}", sessionId);
				proxy.Open();
				sessionId = proxy.InnerChannel.SessionId;
				Console.WriteLine("Session Id after Open() : {0}", sessionId);
				proxy.TestMethod();
				proxy.TestMethod();
			}
		}

		static void NetNamedPipeBindingSessionIdTest()
		{
			Console.WriteLine("\nNetNamedPipeBinding Session Id Test......");

			ChannelFactory<ISessionful> factory = 
				new ChannelFactory<ISessionful>("NetNamedPipeBinding_ISessionful");
			ISessionful proxy = factory.CreateChannel();
			using (proxy as IDisposable) {
				string sessionId = (proxy as IContextChannel).SessionId;
				Console.WriteLine("Session Id before Open() : {0}",
					sessionId);
				proxy.TestMethod();
				proxy.TestMethod();
			}
		}

		static void WSHttpBindingSecureSessionIdTest()
		{
			Console.WriteLine("\nWSHttpBinding Secure Session Id Test......");

			using (SessionfulClient proxy =
						new SessionfulClient("WSHttpBinding_ISessionful")) {
				// 보안 세션은 프록시의 Open 호출 이후에 세션 아이디를 액세스 할 수 있다.
				proxy.Open();
				string sessionId = proxy.InnerChannel.SessionId;
				Console.WriteLine("Session Id after Open() : {0}", sessionId);
				proxy.TestMethod();
				proxy.TestMethod();
			}
		}

		static void WSDualHttpBindingReliableSessionIdTest()
		{
			Console.WriteLine("\nWSDualHttpBinding Reliable Session Id Test......");

			using (SessionfulClient proxy =
						new SessionfulClient("WSDualHttpBinding_ISessionful")) {
				// 신뢰할 수 있는 세션은 Open 호출 이후에 세션 아이디를 액세스 할 수 있지만
				// 그 값은 null 이다. Open 호출 이후에 실제 세션 아이디 값을 알 수 있다.
				string sessionId = proxy.InnerChannel.SessionId;
				Console.WriteLine("Session Id before Open() : {0}", sessionId);
				proxy.Open();
				sessionId = proxy.InnerChannel.SessionId;
				Console.WriteLine("Session Id after Open() : {0}", sessionId);
				proxy.TestMethod();
				proxy.TestMethod();
			}
		}
	}
}
