﻿using System;
using System.Collections.Generic;
using System.Text;
using CallbackTestClient.TestService;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace CallbackTestClient
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Simple Callback Test Client Sample......");
			UsingClientProxy();
			UsingChannelProxy();
		}

		static void UsingClientProxy()
		{
			InstanceContext ctx = new InstanceContext(new CallbackHandler());
			using (TestContractClient proxy = new TestContractClient(ctx)) {
				Console.WriteLine("Before call TestMethod...");
				proxy.TestMethod();
				Console.WriteLine("After call TestMethod...");
			}
		}

		static void UsingChannelProxy()
		{
			InstanceContext ctx = new InstanceContext(new CallbackHandler());
			WSDualHttpBinding binding = new WSDualHttpBinding();
			binding.Security.Mode = WSDualHttpSecurityMode.None;

			ServiceEndpoint ep = new ServiceEndpoint(
				ContractDescription.GetContract(typeof(ITestContract)),
				binding,
				new EndpointAddress("http://loner/wcf/example/callback/test"));
			DuplexChannelFactory<ITestContract> factory = new DuplexChannelFactory<ITestContract>(ctx, ep);
			ITestContract proxy = factory.CreateChannel();
			Console.WriteLine("Before call TestMethod...");
			proxy.TestMethod();
			Console.WriteLine("After call TestMethod...");
			(proxy as IDisposable).Dispose();
		}
	}

	class CallbackHandler : ITestContractCallback
	{
		public void OnCallback(string message)
		{
			Console.WriteLine("Callback: {0}", message);
		}
	}
}
